OtoMarker {
	classvar <markerString = "// **** Marker% ****", <endString = "// **** End% ****";
	classvar <markerNumPoint, <endNumPoint;
	
	*initClass {
		markerNumPoint = markerString.find("%");
		endNumPoint = markerString.find("%");
	}
	
	*run {|num = 1|
		var doc, sString, eString, pos, endpos;
		doc = Document.current;
		
		sString = format(markerString, num);
		pos = doc.string.find(sString);
		
		if(pos.notNil,{
			eString = format(endString, num);
			endpos = doc.string.find(eString, false, pos);
			
			if(endpos.notNil,{
				doc.string.copyRange(pos+markerString.size+1, endpos-1).interpret;
			},{
				doc.string.copyToEnd(pos+markerString.size+1).interpret;
			});
		});
	}
	
	*select {|num = 1|
		var doc, sString, eString, pos, endpos;
		doc = Document.current;
		
		sString = format(markerString, num);
		pos = doc.string.find(sString);
		
		if(pos.notNil,{
			eString = format(endString, num);
			endpos = doc.string.find(eString, false, pos);
			
			if(endpos.notNil,{
				doc.selectRange(pos, endpos + endString.size+1 - pos)
			},{
				doc.selectRange(pos, markerString.size+1)
			});
		});
	}
	
	*remove {|num = 1|
		var doc, sString, eString, pos, endpos;
		doc = Document.current;
		
		sString = format(markerString, num);
		pos = doc.string.find(sString);
		
		eString = format(endString, num);
		endpos = doc.string.find(eString);
		
		if(pos.notNil,{
			doc.string_("", pos, markerString.size+1);
		});
		
		if(endpos.notNil,{
			doc.string_("", endpos - (markerString.size+1), endString.size+1);
		});
		
		if(pos.notNil,{
			if(endpos.notNil,{
				doc.selectRange(pos, endpos - (markerString.size+1) - pos)
			},{
				doc.selectRange(pos, doc.string.size - pos)
			});
		});
	}
	
		
	*insert {|num = 1|
		var doc, sString, eString;
		this.remove(num);
		
		doc = Document.current;
		
		sString = format(markerString, num);
		eString = format(endString, num);
		
		if(doc.selectedString.at(doc.selectionSize-1) == $\n,{
			doc.selectedString_(sString ++ $\n ++ doc.selectedString ++ eString ++ $\n);
		},{
			if(doc.string.at(doc.selectionStart+doc.selectionSize) == $\n,{
				doc.selectedString_(sString ++ $\n ++ doc.selectedString ++ $\n ++ eString);
			},{
				doc.selectedString_(sString ++ $\n ++ doc.selectedString ++ $\n ++ eString ++ $\n);
			});
		});
	}
	
	
	*selectCurrent {
		var doc, eString, pos, endpos, num;
		doc = Document.current;
				
		if( doc.string(doc.selectionStart, markerString.size)
			.containsStringAt(0, markerString.keep(markerNumPoint)), {
			pos = doc.selectionStart;
		},{
			pos = doc.string.findBackwards(markerString.keep(markerNumPoint), false, doc.selectionStart + markerNumPoint - 1);
		});
		
		if(pos.notNil,{
			num = doc.string.at(pos + markerNumPoint).digit;
			
			eString = format(endString, num);
			endpos = doc.string.find(eString, false, pos);
			
			if(endpos.notNil,{
				doc.selectRange(pos, endpos + endString.size+1 - pos)
			},{
				doc.selectRange(pos, markerString.size+1)
			});
		});
	}
	
	
	*runCurrent {
		var doc, eString, pos, endpos, num;
		doc = Document.current;
				
		if( doc.string(doc.selectionStart, markerString.size)
			.containsStringAt(0, markerString.keep(markerNumPoint)), {
			pos = doc.selectionStart;
		},{
			pos = doc.string.findBackwards(markerString.keep(markerNumPoint), false, doc.selectionStart + markerNumPoint - 1);
		});
		
		if(pos.notNil,{
			num = doc.string.at(pos + markerNumPoint).digit;
			
			eString = format(endString, num);
			endpos = doc.string.find(eString, false, pos);
			
			if(endpos.notNil,{
				doc.string.copyRange(pos+markerString.size+1, endpos-1).interpret;
			},{
				doc.string.copyToEnd(pos+markerString.size+1).interpret;
			});
		});
	}
	
	*removeCurrent {
		var doc, eString, pos, endpos, num;
		doc = Document.current;
				
		if( doc.string(doc.selectionStart, markerString.size)
			.containsStringAt(0, markerString.keep(markerNumPoint)), {
			pos = doc.selectionStart;
		},{
			pos = doc.string.findBackwards(markerString.keep(markerNumPoint), false, doc.selectionStart + markerNumPoint - 1);
		});
			
		if(pos.notNil,{
			endpos = doc.string.find(eString, false, pos);
			doc.string_("", pos, markerString.size+1);
		});
		
		if(endpos.notNil,{
			doc.string_("", endpos - (markerString.size+1), endString.size+1);
		});
		
		if(pos.notNil,{
			if(endpos.notNil,{
				doc.selectRange(pos, endpos - (markerString.size+1) - pos)
			},{
				doc.selectRange(pos, doc.string.size - pos)
			});
		});
	}
}


OtoMenu {
	// Main var
	classvar <window, <markerMenues, <keyDownAction;
	
	*initClass {	
		keyDownAction = {| view, char, mod, unicode, keycode |
			var num;
			
			num = switch(keycode,
				18, {1},
				19, {2},
				20, {3},
				21, {4},
				23, {5},
				22, {6},
				26, {7},
				28, {8},
				25, {9}
			);
			
			switch (mod,
				0, {OtoMarker.select(num)},
				524320, {OtoMarker.run(num)},
				131074, {OtoMarker.insert(num)},
				393219, {OtoMarker.remove(num)}
			);
			
		};
	}
	
	*new {
		var winbounds;
		
		if(window.notNil, {
			winbounds = window.bounds;
			window.close;
			{this.createGUI(winbounds)}.defer(0.2);
		},{
			this.createGUI;
		});
	}
	
	*resetAll {
		window = nil; markerMenues = nil;
	}
	
	*createGUI {| winbounds |

		// Reset All
		this.resetAll;
		
		// Create GUI
		
		window = GUI.window.new("OtoMenu", winbounds ? Rect(480, 20, 692, 48), false, true)
			.alpha_(0.85)
			.onClose_({{this.resetAll}.defer(0.1)})
			.alwaysOnTop_(false)
			.front;
		
		window.view
			.background_(Color.gray(0.4))
			.decorator_(FlowLayout(window.view.bounds).margin_(0@0).gap_(4@0))
			.keyDownAction_(keyDownAction);
		
		
		markerMenues = Array.fill(10,
			{|i| var compositer, menu, runButton, jumpButton;
			
			if(i==0, {
				window.view.decorator.gap_(12@0);
				compositer = GUI.compositeView.new(window, Rect(0, 0, 64, 32));
				compositer.decorator_(FlowLayout(compositer.bounds).margin_(0@0).gap_(0@0));
				
				menu = GUI.popUpMenu.new(compositer, Rect(0, 0, 64, 16))
					.items_(["Current",
							 "------", "Remove"])
					.background_(Color.gray(0.5)).font_(Font("Monaco", 10))
					.action_({|me|
						switch (me.items.at(me.value),
							"Remove", {OtoMarker.removeCurrent(i)}
						);
						me.value = 0;
					})
					.canFocus_(false);
				
				compositer.decorator.nextLine;
				
				jumpButton = GUI.button.new(compositer, Rect(0, 0, 32, 16))
					.font_(Font("Monaco", 10))
					.states_([ ["Sel", Color.black, Color.gray(0.7)] ])
					.action_({OtoMarker.selectCurrent})
					.canFocus_(false);
				
				runButton = GUI.button.new(compositer, Rect(0, 0, 32, 16))
					.font_(Font("Monaco", 10))
					.states_([ ["Run", Color.black, Color.gray(0.7)] ])
					.action_({OtoMarker.runCurrent})
					.canFocus_(false);
				
				window.view.decorator.gap_(4@0);
				
			},{	
				compositer = GUI.compositeView.new(window, Rect(0, 0, 64, 32));
				compositer.decorator_(FlowLayout(compositer.bounds).margin_(0@0).gap_(0@0));
				
				menu = GUI.popUpMenu.new(compositer, Rect(0, 0, 64, 16))
					.items_(["Marker"++(i),
							 "------", "Insert", "Remove"])
					.background_(Color.gray(0.7)).font_(Font("Monaco", 10))
					.action_({|me|
						switch (me.items.at(me.value),
							"Insert", {OtoMarker.insert(i)},
							"Remove", {OtoMarker.remove(i)}
						);
						me.value = 0;
					})
					.canFocus_(false);
				
				compositer.decorator.nextLine;
				
				jumpButton = GUI.button.new(compositer, Rect(0, 0, 32, 16))
					.font_(Font("Monaco", 10))
					.states_([ ["Sel", Color.black, Color.gray(0.8)] ])
					.action_({OtoMarker.select(i)})
					.canFocus_(false);
				
				runButton = GUI.button.new(compositer, Rect(0, 0, 32, 16))
					.font_(Font("Monaco", 10))
					.states_([ ["Run", Color.black, Color.gray(0.8)] ])
					.action_({OtoMarker.run(i)})
					.canFocus_(false);
			});
			
			});
		
	}
}